package com.mycompany.app;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Test;
import org.springframework.boot.SpringApplication;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.Is.is;

public class AppTest {

    private static final String COMPONENTS = "components";
    private static final String NANO_TIME = "nanoTime";
    private static final String DETAILS = "details";


    @Test
    public void testHealthEndpointCached() {
        final SpringApplication app = new SpringApplication(App.class);
        final RestTemplate restTemplate = new RestTemplate();
        try (final ConfigurableApplicationContext ignored = app.run()) {
            final String url = "http://localhost:8080/actuator/health";
            final ResponseEntity<JsonNode> responseEntity = restTemplate.getForEntity(url, JsonNode.class);
            assertThat(responseEntity.getStatusCode().is2xxSuccessful(), is(true));
            final ResponseEntity<JsonNode> responseEntityCached = restTemplate.getForEntity(url, JsonNode.class);
            assertThat(responseEntityCached.getStatusCode().is2xxSuccessful(), is(true));
            final long nanoTime = getNanoTimeFromHealthEndpoint(responseEntity.getBody());
            final long nanoTimeCached = getNanoTimeFromHealthEndpoint(responseEntityCached.getBody());
            assertThat("Health endpoint response should be cached, nanoTime for both responses should be same", nanoTimeCached, is(nanoTime));
        }
    }

    private long getNanoTimeFromHealthEndpoint(final JsonNode rootNode) {
        // in 2.2.0+ json schema changed to use `components` attribute instead of `details`
        if (rootNode.get(COMPONENTS) != null) {
            return rootNode.get(COMPONENTS).get(NANO_TIME).get(DETAILS).get(NANO_TIME).longValue();
        } else {
            return rootNode.get(DETAILS).get(NANO_TIME).get(DETAILS).get(NANO_TIME).longValue();
        }
    }
}
